@@ -254,11 +254,13 @@ class Test_add_handler(unittest.TestCase):
         config = self._makeOne()
         def dummy_add_view(**kw):
             views.append(kw)
+        class DummyHandler(object):
+            def one(self): pass
         config.add_view = dummy_add_view
-        config.add_handler('name', '/abc', 'pyramid')
+        config.add_handler('name', '/abc', DummyHandler)
         self.assertEqual(len(views), 1)
         view = views[0]
-        self.assertEqual(view['view'], pyramid)
+        self.assertEqual(view['view'], DummyHandler)
 
     def test_add_handler_pattern_None_no_previous_route(self):
         from pyramid.exceptions import ConfigurationError
@@ -273,13 +275,88 @@ class Test_add_handler(unittest.TestCase):
         views = []
         def dummy_add_view(**kw):
             views.append(kw)
+        class DummyHandler(object):
+            def one(self): pass
         config.add_view = dummy_add_view
         config.add_route = None # shouldn't be called
-        config.add_handler('name', None, 'pyramid')
+        config.add_handler('name', None, DummyHandler)
         self.assertEqual(len(views), 1)
         view = views[0]
-        self.assertEqual(view['view'], pyramid)
+        self.assertEqual(view['view'], DummyHandler)
+    
+    def test_add_handler_explicit_action_lacking(self):
+        import pyramid
+        config = self._makeOne()
+        config.add_route('name', ':def')
+        views = []
+        def dummy_add_view(**kw): views.append(kw)
+        class DummyHandler(object):
+            def one(self): pass
+        config.add_view = dummy_add_view # shouldn't be called
+        config.add_route = None # shouldn't be called
+        config.add_handler('name', None, DummyHandler, action='two')
+        self.assertEqual(len(views), 0)
 
+    def test_add_handler_explicit_action_and_extra_exposed(self):
+        import pyramid
+        config = self._makeOne()
+        config.add_route('name', ':def')
+        views = []
+        def dummy_add_view(**kw): views.append(kw)
+        class DummyHandler(object):
+            def two(self): pass
+            two.__exposed__ = [{'name':'one'}]
+        config.add_view = dummy_add_view # shouldn't be called
+        config.add_route = None # shouldn't be called
+        config.add_handler('name', None, DummyHandler, action='two')
+        self.assertEqual(len(views), 1)
+        view = views[0]
+        self.assertEqual(view['view'], DummyHandler)
+
+    def test_add_handler_with_view_permission_and_action_in_path(self):
+        from pyramid_handlers import action
+        config = self._makeOne()
+        views = []
+        def dummy_add_view(**kw):
+            views.append(kw) # pragma: no cover
+        config.add_view = dummy_add_view
+        class MyView(DummyHandler):
+            @action(permission='different_perm')
+            def action_with_non_default_permission(self): # pragma: no cover
+                return 'My permission is different!'
+        config.add_handler('name', '/{action}', MyView, view_permission='perm')
+        self._assertRoute(config, 'name', '/{action}', 0)
+        self.assertEqual(len(views), 3)
+        for view in views:
+            self.assert_('permission' in view)
+            if view['attr'] == 'action_with_non_default_permission':
+                self.assertEqual(view['permission'], 'different_perm')
+            else:
+                self.assertEqual(view['permission'], 'perm')
+
+    def test_add_handler_with_view_permission_and_action_as_kwarg(self):
+        from pyramid_handlers import action
+        config = self._makeOne()
+        views = []
+        def dummy_add_view(**kw):
+            views.append(kw) # pragma: no cover
+        config.add_view = dummy_add_view
+        class MyView(DummyHandler):
+            def index(self): # pragma: no cover
+                return 'Index'
+            @action(name='index', permission='different_perm')
+            def index2(self): # pragma: no cover
+                return 'Index with different permission.'
+        config.add_handler('name', '/', MyView, action='index',
+                           view_permission='perm')
+        self._assertRoute(config, 'name', '/', 0)
+        self.assertEqual(len(views), 2)
+        for view in views:
+            self.assert_('permission' in view)
+            if view['attr'] == 'index':
+                self.assertEqual(view['permission'], 'perm')
+            elif view['attr'] == 'index2':
+                self.assertEqual(view['permission'], 'different_perm')
 
     def _assertRoute(self, config, name, path, num_predicates=0):
         from pyramid.interfaces import IRoutesMapper
@@ -440,8 +517,7 @@ class TestHandlerDirective(unittest.TestCase):
 
         route_action = actions[0]
         route_discriminator = route_action['discriminator']
-        self.assertEqual(route_discriminator,
-                         ('route', 'name', False, None, None, None, None,None))
+        self.assertEqual(route_discriminator[:2], ('route', 'name'))
         self._assertRoute('name', '/:action')
 
         view_action = actions[1]
